<?php

class Cloudns_Actions {
	/**
	 * @var core 
	 */
	protected $core;
	protected $params;

	public function __construct ($params) {
		$this->core = Cloudns_Core::inst($params);
		$this->params = $params;
	}
	
	/**
	 * Gets the zones from the module's table
	 * @return array
	 */
	public function getZones () {
		
		if ($this->params['registeredDomains'] == 'on') {
			
			$zones = $this->core->Controller->getRegisteredDomains();
			foreach ($zones as &$zone) {
				$zone['ascii'] = $this->core->Helper->getUnicodeName($zone['name']);
			}
			
			$response = $zones;
		} else {
			$response = $this->core->Controller->getZones();
		}
		return array(
			'response' => $response,
			'zones' => $response,
			'cloudAction' => 'zones',
			'version' => $this->params['shortVersion'],
			'theme' => $this->params['theme'],
		);
	}
	
	/**
	 * Gets the info needed to display the Add new record page
	 * @param array $zoneInfo
	 * @param string $recordType
	 * @return array
	 */
	public function addNewRecord ($zoneInfo, $recordType) {
		if (!empty($zoneInfo)) {
			
			if ($zoneInfo['zone'] == 'ipv4' || $zoneInfo['zone'] == 'ipv6') {
				$zone_type = 'reverse';
			} else {
				$zone_type = $zoneInfo['zone'];
			}
			
			$recordTypes = $this->core->Controller->getRecordTypes($zone_type);
			$ttl = $this->core->Controller->getAvailableTTL();
			$type = 'A';
			if (in_array($recordType, $recordTypes)) {
				$type = $recordType;
			}

			$shortName = $this->core->Helper->shortenLongName($zoneInfo['name'], 34);

			return array(
				'pagetitle' => 'Add new record for '.$zoneInfo['name'],
				'response' => $response,
				'recordTypes' => $recordTypes,
				'defaultType' => $type,
				'shortName' => $shortName,
				'ttls' => $ttl,
				'settings' => array(),
				'cloudAction' => 'add-new-record',
				'version' => $this->params['shortVersion'],
				'theme' => $this->params['theme'],
			);

		} else {
			return array('response' => $response);
		}
	}
	
	/**
	 * Adds new record
	 * @param array $zoneInfo
	 * @param string $recordType The type of the new record
	 * @param array $settings The record's settings submited by the customer
	 * @return array
	 */
	// todo
	public function doAddNewRecord ($zoneInfo, $recordType, $settings) {
		$zone = $zoneInfo['name'];
		
		$recordTypes = $this->core->Controller->getRecordTypes($zoneInfo['zone']);
		$ttl = $this->core->Controller->getAvailableTTL();
		
		$response = $this->core->Records->recordAdd($zone, $recordType, $settings);

		if ($response['status'] == 'success') {
			$this->core->Helper->redirect("clientarea.php?action=productdetails&id={$this->params['serviceid']}&customAction=zone-settings&zone={$zone}");
		}
		$shortName = $this->core->Helper->shortenLongName($zone, 34);

		return array(
			'pagetitle' => 'Add new record for '.$zone,
			'response' => $response,
			'recordTypes' => $recordTypes,
			'defaultType' => $recordType,
			'settings' => $settings,
			'zone' => $zone,
			'ttls' => $ttl,
			'shortName' => $shortName,
			'cloudAction' => 'add-record',
			'version' => $this->params['shortVersion'],
			'theme' => $this->params['theme'],
		);
	}
	
	/**
	 * Gets the information needed to display the Edit record page
	 * @param array $zoneInfo
	 * @param int $record_id ID of the record in the ClouDNS system
	 * @return array
	 */
	public function editRecord ($zoneInfo, $record_id) {
		$record = $this->core->Controller->getRecordById($zoneInfo['name'], $record_id);
			
		if (empty($record)) {
			$this->core->Helper->redirect("clientarea.php?action=productdetails&id={$this->params['serviceid']}&customAction=zone-settings&zone={$zoneInfo['name']}");
		}

		$record['full_host'] = $this->core->Helper->getUnicodeName($record['host'] . (mb_strlen($record['host']) > 0 ? '.':'') . $zoneInfo['name']);
		$shortName = $this->core->Helper->shortenLongName($zoneInfo['name'], 34);
		$uniHost = $this->core->Helper->getUnicodeName($record['host']);
		
		$ttl = $this->core->Controller->getAvailableTTL();
		
		return array(
			'pagetitle' => 'Edit record',
			'response' => $response,
			'shortName' => $shortName,
			'record' => $record,
			'uniHost'=>$uniHost,
			'zone' => $zoneInfo['name'],
			'ttls' => $ttl,
			'cloudAction' => 'edit-record',
			'version' => $this->params['shortVersion'],
			'theme' => $this->params['theme'],
		);
	}
	
	/**
	 * 
	 * @param array $zoneInfo
	 * @param int $record_id
	 * @param array $settings
	 * @param int $status
	 * @param string $recordType
	 * @return array
	 */
	public function doEditRecord ($zoneInfo, $record_id, $settings, $status, $recordType) {
		$record = $this->core->Controller->getRecordById($zoneInfo['name'], $record_id);
		$uniHost = $this->core->Helper->getUnicodeName($record['host']);
		$shortName = $this->core->Helper->shortenLongName($zoneInfo['name'], 34);
		
		$response = $this->core->Records->recordEdit($zoneInfo['name'], $record_id, $settings, $status);
			
		// merging the arrays so the new values can be used to fill
		// the form, if there is an error with the record saving 
		$settings['fp_type'] = $this->core->Helper->getPost('fp_type');
		$record = array_merge($record, $settings);

		if ($response['status'] == 'success') {
			$this->core->Helper->redirect("clientarea.php?action=productdetails&id={$this->params['serviceid']}&customAction=zone-settings&zone={$zoneInfo['name']}");
		}

		return array(
			'pagetitle' => 'Edit record',
			'response' => $response,
			'shortName' => $shortName,
			'record' => $record,
			'uniHost'=>$uniHost,
			'settings' => $settings,
			'zone' => $zoneInfo['name'],
			'cloudAction' => 'do-edit-record',
			'version' => $this->params['shortVersion'],
			'theme' => $this->params['theme'],
		);
	}
	
	/**
	 * Gets the SOA settings
	 * @param array $zoneInfo
	 * @return array
	 */
	public function getSOASettings ($zoneInfo) {
		if (!empty($zoneInfo)) {
			$soa = $this->core->SOA->getSOA($zoneInfo['name']);
			
			if (empty($soa)) {
				$response = array('status'=>'error', 'description'=>'It looks like there is no such zone');
			}
			
			$unicodePrymaryNS = $this->core->Helper->getUnicodeName($soa['primaryNS']);
			
			return array(
				'pagetitle' => 'SOA settings of '.$zoneInfo['name'],
				'response' => $response,
				'soa' => $soa,
				'primaryNS' => $unicodePrymaryNS,
				'cloudAction' => 'soa-settings',
				'version' => $this->params['shortVersion'],
				'theme' => $this->params['theme'],
			);
		} else {
			return array('response' => 'An error occured');
		}
	}
	
	/**
	 * 
	 * @param array $zoneInfo
	 * @param array $soa
	 * @param string $soaAction
	 * @return array
	 */
	public function editSOA ($zoneInfo, $soa, $soaAction) {
		$zone = $zoneInfo['name'];
		if (!empty($zoneInfo)) {

			if ($soaAction == 'do_save') {
				$response = $this->core->SOA->editSOA($zone, $soa['primaryNS'], $soa['adminMail'], $soa['refresh'], $soa['retry'], $soa['expire'], $soa['defaultTTL']);
			} elseif ($soaAction == 'do_reset') {
				$response = $this->core->SOA->resetSOA($zone);
			} else {
				$response = array('status'=>'error', 'description'=>'An error occurred while saving the changes!');
			}

			if (isset($response['status']) && $response['status'] == 'error') {
				$soaSettings = $this->core->SOA->getSOA($zone);
				$soa = array_merge($soaSettings, $soa);
			} else {
				$this->core->Helper->redirect("clientarea.php?action=productdetails&id={$this->params['serviceid']}&customAction=soa-settings&zone={$zone}");
			}

			$unicodePrymaryNS = $this->core->Helper->getUnicodeName($soa['primary_ns']);
			return array(
				'pagetitle' => 'SOA settings edit of '.$zone,
				'response' => $response,
				'zoneInfo' => $zoneInfo,
				'soa' => $soa,
				'cloudAction' => 'edit-soa-settings',
				'version' => $this->params['shortVersion'],
				'theme' => $this->params['theme'],
			);
		} else {
			return array('response' => $response);
		}
	}
	
	/**
	 * Calls a method depending on the zone type (slave or master)
	 * @param array $zoneInfo
	 * @param string $type Record type to return only these records for master zones
	 * @return array
	 */
	public function getSettings ($zoneInfo, $type, $recordTypes) {
		if (!empty($zoneInfo)) {
			if ($zoneInfo['type'] == 'master') {
				return $this->getRecords($zoneInfo, $type, $recordTypes);
			} else {
				return $this->getMasterServers($zoneInfo);
			}
		} else {
			return array('response' => 'There is no such DNS zone');
			
		}
	}
	
	/**
	 * 
	 * @param array $zoneInfo
	 * @param string $type
	 * @param array
	 * @return array
	 */
	public function getRecords ($zoneInfo, $type, $recordTypes, $response=array()) {
		$records = $this->core->Controller->getRecords($zoneInfo['name'], $type);

		foreach ($records as &$row) {
			$row['full_host'] = $this->core->Helper->getUnicodeName($row['host'] . (mb_strlen($row['host']) > 0 ? '.':'') . $zoneInfo['name']);
			$row['shortHost'] = $this->core->Helper->shortenLongName($row['host'] . (mb_strlen($row['host']) > 0 ? '.':'').$zoneInfo['name'], 24);

			if ($row['type'] == 'MX') {
				$row['priority'] = $row['priority'].' ';
			} 
			if ($row['type'] == 'SRV') {
				$row['priority'] = $row['priority'].' '.$row['weight'].' '.$row['port'].' ';
			} 
			if (in_array($row['type'], array('SSHFP', 'TXT', 'SPF', 'MX', 'SRV'))) {
				$row['shortRecord'] = $this->core->Helper->shortenLongString($row['priority'].$row['record'], 24);
			} elseif ($row['type'] == 'RP') {
				$row['shortRecord'] = $this->core->Helper->shortenLongName($row['mail'], 24);

			} elseif ($row['type'] == 'NAPTR') {
				$row['shortRecord'] = $this->core->Helper->shortenLongName($row['order'].' '.$row['pref'].' "'.$row['flag'].'" "'.$row['params'].'" '.(!empty($row['regexp']) ? '"'.$row['regexp'].'"' : '""').' '.(!empty($row['replace']) ? $row['replace'] : '.'), 30);
			} elseif ($row['type'] == 'CAA') {
				$row['shortRecord'] = $this->core->Helper->shortenLongName($row['caa_flag'].' '.$row['caa_type'].' "'.$row['caa_value'].'"' , 30);
			} else {
				$row['shortRecord'] = $this->core->Helper->shortenLongName($row['record'], 24);
			}

			$row['ttl_seconds'] = $this->core->Helper->convertSeconds($row['ttl']);
		}

		return array(
			'pagetitle' => 'DNS records of '.$zoneInfo['name'],
			'zoneInfo' => $zoneInfo,
			'recordTypes' => $recordTypes,
			'defaultType' => $type,
			'records' => $records,
			'cloudAction' => 'zone-settings',
			'version' => $this->params['shortVersion'],
			'theme' => $this->params['theme'],
			'response' => $response,
		);
	}
	
	/**
	 * Deletes a record
	 * @param array $zoneInfo
	 * @param int $record_id
	 * @return type
	 */
	public function deleteRecord ($zoneInfo, $record_id) {
		$response = $this->core->Records->delete($zoneInfo['name'], $record_id);
		if ($response['status'] == 'success') {
			$this->core->Helper->redirect("clientarea.php?action=productdetails&id={$this->params['serviceid']}&customAction=zone-settings&zone={$zoneInfo['name']}");
		}
		
		return $this->getRecords($zoneInfo, 'all', $response);
	}
	
	/**
	 * Deletes a zone
	 * @param array $zoneInfo
	 * @return array
	 */
	public function deleteZone ($zone) {
		$response = $this->core->Controller->deleteZone($zone);
		
		if ($response['status'] == 'success') {
			$this->core->Helper->redirect("clientarea.php?action=productdetails&id={$this->params['serviceid']}&customAction=zones");
		}
		$toReturn = $this->getZones();
		$toReturn['response'] = $response['description'];
		
		return $toReturn;
	}
	
	/**
	 * Gets the slave servers for slave zones
	 * @param array $zoneInfo
	 * @return array
	 */
	public function getBINDSettings ($zoneInfo) {
		$ipv4 = '';
		$ipv6 = '';
		foreach ($this->core->Servers->getMasterServers() as $server) {
			$ipv4 .= "		{$server['ip4']};\n";
			if (!is_null($server['ip6'])) {
				$ipv6 .= "		{$server['ip6']};\n";
			}
		}
		
		return array(
			'pagetitle' => 'Example Bind settings of '.$zoneInfo['name'],
			'response' => $response,
			'zoneInfo' => $zoneInfo,
			'records' => $records,
			'ipv4' => $ipv4,
			'ipv6' => $ipv6,
			'cloudAction' => 'bind-settings',
			'version' => $this->params['shortVersion'],
			'theme' => $this->params['theme'],
		);
	}
	
	/**
	 * Gets the master servers of a slave zone, added by the client
	 * @param array $zoneInfo
	 * @return array
	 */
	public function getMasterServers ($zoneInfo) {
		$masterServers = $this->core->Slave->getMasterServers($zoneInfo['name']);
		$response = '';
		return array(
			'pagetitle' => 'Master servers of '.$zoneInfo['name'],
			'zoneInfo' => $zoneInfo,
			'response' => $response,
			'masterServers' => $masterServers,
			'cloudAction' => 'zone-settings',
			'version' => $this->params['shortVersion'],
			'theme' => $this->params['theme'],
		);
	}
	
	/**
	 * Deletes a master server
	 * @param string $zone
	 * @param string $master
	 * @return array
	 */
	public function deleteMasterServer ($zone, $master) {
		$response = $this->core->Slave->deleteMasterServer($zone, $master);
		if ($response['status'] == 'success') {
			$this->core->Helper->redirect("clientarea.php?action=productdetails&id={$this->params['serviceid']}&customAction=zone-settings&zone={$zone}");
		}
		$masterServers = $this->core->Slave->getMasterServers($zone);
		return array(
			'pagetitle' => 'Master servers of '.$zone,
			'response' => $response,
			'masterServers' => $masterServers,
			'cloudAction' => 'delete-master-server',
			'version' => $this->params['shortVersion'],
			'theme' => $this->params['theme'],
		);
	}
	
	/**
	 * 
	 * @param string $zone
	 * @param string $master
	 * @return array
	 */
	public function addMasterServer ($zone, $master) {
		$response = $this->core->Slave->addMasterServer($zone, $master);
		if ($response['status'] == 'success') {
			$this->core->Helper->redirect("clientarea.php?action=productdetails&id={$this->params['serviceid']}&customAction=zone-settings&zone={$zone}");
		}
		$masterServers = $this->core->Slave->getMasterServers($zone);
		
		return array(
			'pagetitle' => 'Master servers of '.$zone,
			'response' => $response,
			'masterServers' => $masterServers,
			'cloudAction' => 'add-master-server',
			'version' => $this->params['shortVersion'],
			'theme' => $this->params['theme'],
		);
	}
	
	/**
	 * Gets the update status of a zone
	 * @param array $zoneInfo
	 * @return array
	 */
	public function getUpdateStatus ($zoneInfo) {
		$updated = $this->core->Zones->getUpdateStatus($zoneInfo['name']);
		return array(
			'pagetitle' => 'Update status of '.$zoneInfo['name'],
			'response' => '',
			'updated' => $updated,
			'zoneInfo' => $zoneInfo,
			'cloudAction' => 'update-status',
			'version' => $this->params['shortVersion'],
			'theme' => $this->params['theme'],
		);
	}
	
	/**
	 * Updates the zone at the DNS servers
	 * @param array $zoneInfo
	 * @return array
	 */
	public function updateZone ($zoneInfo) {
		$response = $this->core->Zones->updateZone($zoneInfo['name']);
		if ($response['status'] == 'success') {
			$this->core->Helper->redirect("clientarea.php?action=productdetails&id={$this->params['serviceid']}&customAction=update-status&zone={$zoneInfo['name']}");
		} else {
			$updated = $this->core->Zones->getUpdateStatus($zoneInfo['name']);
			return array(
				'pagetitle' => 'Update status of '.$zoneInfo['name'],
				'response' => $response,
				'updated' => $updated,
				'zoneInfo' => $zoneInfo,
				'cloudAction' => 'update',
				'version' => $this->params['shortVersion'],
				'theme' => $this->params['theme'],
			);
		}
	}
	
	/**
	 * Imports records from a text area in BIND or TinyDNS format
	 * @param string $zone
	 * @param string $records A list with all the records submited by the client
	 * @param int $delete 1 or 0
	 * @param string $format BIND or TinyDNS
	 * @return array
	 */
	public function importRecords ($zone, $records, $delete, $format) {
		$response = $this->core->Records->import($zone, $records, $format, $delete);
		
		if ($response['status'] == 'success') {
			$this->core->Helper->redirect("clientarea.php?action=productdetails&id={$this->params['serviceid']}&customAction=zone-settings&zone={$zone}");
		}
		return array(
			'pagetitle' => 'Import records to '.$zone,
			'response' => $response,
			'cloudAction' => 'import',
			'version' => $this->params['shortVersion'],
			'theme' => $this->params['theme'],
		);
	}
	
	/**
	 * Returns the data needed to display the statistics
	 * @param array $zoneInfo
	 * @param string $date
	 * @return array
	 */
	public function getStats ($zoneInfo, $date) {
		$zone = $zoneInfo['name'];
		$stats = $this->core->Statistics->getZoneStats($zone, $date);
		
		$requests = array_sum($stats);
		$statsTable = '';
		foreach ($stats as $key => $value) {
			$showLink = true;

			if ($date == 'last-30-days') {
				$unixtime = strtotime($key);
				$timeHTML = date('l, F d, Y', $unixtime);
				$timeLink = date('Ymd', $unixtime);
				$hover = date('Y-m-d', $unixtime);

				if (strtotime($hover) < strtotime('-1 month')) {
					$showLink = false;
				}
			} else if (strlen($date) == 8) {
				$currhour = $key;
				$nexthour = $currhour + 1;

				if ($nexthour < 10) {
					$nexthour = "0{$nexthour}";
				}

				$timeLink = 0;

				$timeHTML = "{$currhour}:00 - {$nexthour}:00 , ".date('F d , Y', strtotime($date));
				$hover = date('Y-m-d', strtotime($date))." $currhour:00 - $nexthour:00";
				$showLink = false;

			} elseif (strlen($date) == 6) {
				$timeHTML = date('l, F d, Y', strtotime($date . $key));
				$timeLink = $date . $key;
				$hover = date('Y-m-d', strtotime($date . $key));
				if (strtotime($hover) < strtotime('-1 month')) {
					$showLink = false;
				}
				$noLinkDate = date('Ymd', strtotime('-1 month'));

			} elseif (strlen($date) == 4) {
				$timeHTML = date('F, Y', strtotime($date . $key .'01'));
				$timeLink = $date . $key;
				$hover = date('Y-m', strtotime($date . $key .'01'));
			} else {
				$hover = $timeHTML = $timeLink = $key;
			}
			
			// html
			if ($requests == 0) {
				$percent = 0;
			} else {
				$percent = ($value/$requests)*100;
			}

			$tdWidth = strlen($date) * 42;
			if (strlen($date) == 12) {
				$tdWidth = 270;
			} elseif (strlen($date) == 0) {
				$tdWidth = 100;
			}

			$statsTable .= '<tr '.(($showLink == true || strlen($date) < 6) ? 'class="pointer" onClick="javascript: location.href=\'clientarea.php?action=productdetails&id='.$this->params['serviceid'].'&customAction=statistics&zone='.$zone.'&date='.$timeLink.'\'"':'').'>
				<td class="text-right" style="width:'.$tdWidth.'px;">'.$timeHTML.'</td>
				<td>
					<div style="height:10px; width:'.round((450*$percent)/100).'px; background-color:#ffa900; margin:3px 5px 0 5px;" class="pull-left">&nbsp</div> '.number_format($percent, 2).'% ('.number_format($value).' Requests)
				</td>
			</tr>';
		}

		$statLinks = '';
		if (strlen($date) == 8) {
			$statLinks = ' » <a href="clientarea.php?action=productdetails&id='.$this->params['serviceid'].'&customAction=statistics&zone='.$zone.'&date='.substr($date, 0, 4).'">'.substr($date, 0, 4).'</a> » <a href="clientarea.php?action=productdetails&id='.$this->params['serviceid'].'&customAction=statistics&zone='.$zone.'&date='.substr($date, 0, 6).'">'.date('F', strtotime($date)).'</a> » '.date('l d', strtotime(substr($date,0,8)));
		} elseif (strlen($date) == 6) {
			$statLinks = ' » <a href="clientarea.php?action=productdetails&id='.$this->params['serviceid'].'&customAction=statistics&zone='.$zone.'&date='.substr($date, 0, 4).'">'.substr($date, 0, 4).'</a> » '.date('F', strtotime($date.'01'));
		} elseif (strlen($date) == 4) {
			$statLinks = ' » '.substr($date, 0, 4);
		}
		
		$response = '';
		return array(
			'pagetitle' => 'Statistics of '.$zone,
			'response' => $response,
			'links' => $links,
			'statsTable' => $statsTable,
			'statLinks' => $statLinks,
			'requests' => $requests,
			'date' => $date,
			'zoneInfo' => $zoneInfo,
			'cloudAction' => 'statistics',
			'version' => $this->params['shortVersion'],
			'theme' => $this->params['theme'],
			
		);
	}
	
	public function suspendZones () {
		$zones = $this->core->Controller->getZones();
		
		if (!isset($zones['status'])) {
			$message = array();
			foreach ($zones as $zone) {
				$response = $this->core->Zones->suspend($zone['name']);
				if (isset($response['status']) && $response['status'] == 'error') {
					$message[] = $response['description'];
				}
			}

			if (!empty($message)) {
				return implode(', ', $message);
			}
		}
		
		return 'success';
	}
	
	public function unSuspendZones () {
		$zones = $this->core->Controller->getZones();
		
		if (!isset($zones['status'])) {
			$message = array();
			foreach ($zones as $zone) {
				$response = $this->core->Zones->unSuspend($zone['name']);
				if (isset($response['status']) && $response['status'] == 'error') {
					$message[] = $response['description'];
				}
			}
			if (!empty($message)) {
				return implode(', ', $message);
			}
		}
		
		return 'success';
	}
	
	public function terminateAccount () {
		$zones = $this->core->Controller->getZones();
		if (!isset($zones['status'])) {
			$message = array();
			foreach ($zones as $zone) {
				$this->core->Controller->deleteZone($zone['name']);
			}
		}
		
		return 'success';
	}
}
